<template>
  <div>
    <el-bus-crud
      ref="crud"
      v-bind="tableConfig"
      :extra-query="{ levelDown: checkedLevelDown }"
    >
      <template #header>
        <el-checkbox
          v-model="checkedLevelDown"
          true-label="1"
          false-label=""
          style="float: right"
          >未处理的质检异常订单</el-checkbox
        >
      </template>
    </el-bus-crud>
    <div id="print-container">
      <print-list ref="printList" :order-list="orderList" />
    </div>
  </div>
</template>

<script>
import printJS from 'print-js'
import { getPartnerListConfig } from '@/utils/form-item-config'
import { dateRangeOptions } from '@/utils/options'
import CustomDateRange from '@/components/custom-date-range.vue'
import PrintList from '@/components/order/print-list'
export default {
  components: {
    PrintList,
  },
  data() {
    return {
      checkedLevelDown: '',
      orderList: [],
      tableConfig: {
        url: 'flower/api/order/list',
        editUrl: 'flower/api/order/page/update',
        hasNew: false,
        hasEdit: true,
        hasDelete: false,
        persistSelection: true,
        hasExport: true,
        exportUrl: 'flower/api/order/list/export',
        exportText: '导出订单',
        onResetView: (row) => {
          this.$router.push(`${this.$route.path}/${row.id}`)
        },
        operationAttrs: {
          width: 150,
          fixed: 'right',
        },
        beforeRequest: (params) => {
          const searchForm = this.$refs.crud?.$refs?.searchForm
          if (searchForm) {
            const statusComp = searchForm.getComponentById('statusBackend')
            if (statusComp) {
              // 组件内部不会修改这个prop直接修改可忽略警告
              statusComp.extraQuery = { ...params, statusBackend: '' }
              statusComp.getOtherOptions()
            }
          }
        },
        beforeOpen: (row, isNew) => {
          if (!isNew) {
            row.districtStr = this.getDistrict(row)
          }
        },
        columns: [
          { label: '', type: 'selection', minWidth: 60, fixed: 'left' },
          { label: '订单号', prop: 'orderNo', minWidth: 150, fixed: 'left' },
          { label: '用户账号', prop: 'createName', minWidth: 120 },
          { label: '收货人', prop: 'customer', minWidth: 120 },
          { label: '收货人电话', prop: 'customerTel', minWidth: 120 },
          {
            label: '收货地址',
            formatter: (row) =>
              `${row.customerProvince || ''}${row.customerCity || ''}${
                row.customerRegion || ''
              }${row.customerAddress || ''}`,
            minWidth: 250,
          },
          { label: '订单金额(元)', prop: 'totalAmount', minWidth: 150 },
          { label: '底价(元)', prop: 'supplierAmount', minWidth: 150 },
          { label: '订单状态', prop: 'statusBackendStr', minWidth: 120 },
          { label: '下单时间', prop: 'createTime', minWidth: 180 },
          { label: '支付时间', prop: 'paymentTime', minWidth: 180 },
          { label: '合伙人', prop: 'partnerName', minWidth: 120 },
          { label: '库位', prop: 'warehouseLocationCode', minWidth: 120 },
          { label: '特殊需求', prop: 'specialNeedsStr', minWidth: 120 },
          {
            label: '备注',
            formatter: (row) => (
              <simple-text value={row.remarks} type="primary"></simple-text>
            ),
            minWidth: 200,
          },
        ],
        searchFormAttrs: {
          labelWidth: 'auto',
        },
        searchForm: [
          {
            type: 'row',
            id: 'row',
            items: [
              {
                label: '订单状态：',
                id: 'statusBackend',
                type: 'bus-radio',
                el: {
                  hasAll: true,
                  otherInterfaceUri: 'flower/api/order/status/count',
                  childType: 'el-radio-button',
                  filterOptions: (list) => {
                    return list.map((item) => {
                      return {
                        ...item,
                        label: `${item.label}(${item.orderCount})`,
                      }
                    })
                  },
                  // code: 'ORDER_STATUS_BACKEND',
                },
                default: '',
                span: 24,
                searchImmediately: true,
                on: {
                  optionsChange: (e, updateForm, { currentComp }) => {
                    if (Array.isArray(e[0]) && e[0].length > 0) {
                      const totalCount = e[0].reduce((total, current) => {
                        return (total += current.orderCount)
                      }, 0)
                      // 组件内部不会修改这个prop直接修改可忽略警告
                      currentComp.props = {
                        allLabel: `不限(${totalCount})`,
                      }
                    }
                  },
                },
              },
              {
                label: '下单日期：',
                id: 'dateType',
                component: CustomDateRange,
                el: {
                  options: dateRangeOptions,
                },
                searchImmediately: true,
                commonFormat: true,
                commonFormatProps: [
                  'dateType',
                  'createStartDateStr',
                  'createEndDateStr',
                ],
                span: 24,
              },
              { label: '订单号：', id: 'orderNo', type: 'input' },
              { ...getPartnerListConfig() },
              { label: '商品名称：', id: 'flowerName', type: 'input' },
              {
                label: '收货人：',
                id: 'customer',
                type: 'input',
                el: { placeholder: '收货人姓名/手机号' },
              },
              { label: '收货地址：', id: 'address', type: 'input' },
              { label: '库位：', id: 'warehouseLocationCode', type: 'input' },
            ],
          },
        ],
        form: [
          {
            type: 'row',
            span: 12,
            items: [
              {
                label: '地区：',
                id: 'customerProvince',
                component: 'el-bus-select-area',
                span: 24,
                commonFormat: true,
                commonFormatProps: [
                  'customerProvince',
                  'customerCity',
                  'customerRegion',
                ],
                str: true,
                strKey: 'districtStr',
                rules: { required: true, message: '请选择地区' },
              },
              {
                label: '详细地址：',
                id: 'customerAddress',
                type: 'input',
                span: 24,
                rules: {
                  required: true,
                  message: '请输入详细地址且最大不超过50个汉字',
                  trigger: 'blur',
                  max: 50,
                },
              },
              {
                label: '联系方式：',
                id: 'customerTel',
                type: 'input',
                rules: {
                  required: true,
                  pattern: this.$elBusUtil.REG.MOBILEPHONE,
                  message: '请输入合法的手机号',
                  trigger: 'blur',
                },
              },
            ],
          },
        ],
        headerButtons: [
          {
            text: '打印订单',
            type: 'primary',
            atClick: async (selected) => {
              const formValue = this.$refs.crud.$refs.searchForm.getFormValue()
              if (
                !formValue.createStartDateStr &&
                !formValue.createEndDateStr
              ) {
                this.$message.warning('请先选择需要打印订单的日期')
                return false
              }
              if (formValue.createStartDateStr !== formValue.createEndDateStr) {
                this.$message.warning('只能打印某一天的订单')
                return false
              }
              const ids = selected.map((i) => i.id)
              const { code, data } = await this.$elBusHttp.request(
                'flower/api/order/check/location/list',
                {
                  method: 'post',
                  data: {
                    ...formValue,
                    ids: ids?.length ? ids.join(',') : null,
                  },
                }
              )
              if (code === 0) {
                const orderList = data || []
                if (orderList.length !== 0) {
                  this.orderList = orderList
                  // 这边嵌套两个nextTick是因为重写el-table时在nextTick中进行了dom操作
                  this.$nextTick(() => {
                    this.$nextTick(() => {
                      const trs = document.querySelectorAll(
                        '.el-table__footer tr'
                      )
                      if (trs && trs.length > 0) {
                        for (const tr of trs) {
                          const tds = tr.querySelectorAll('td')
                          if (tds && tds.length > 0) {
                            tds[0].colSpan = 8
                            tds[0].style.textAlign = 'center'
                          }
                        }
                      }
                      this.$nextTick(async () => {
                        await printJS({
                          printable: 'print-container',
                          type: 'html',
                          header: '订单明细',
                          headerStyle:
                            'text-align:center;font-size:24px;color:#333;font-weight:bold;margin-bottom:15px',
                          style: `@media print { @page {size: auto; margin: 20mm 0 10mm 0; } body{margin:10mm} .el-table .el-table__header-wrapper{display: none} .el-table .el-table__body-wrapper thead{display: table-header-group} .break-page {page-break-after:always}}`,
                          css: [
                            `${this.$config.baseUrl}print/index.css`,
                            `${this.$config.baseUrl}print/custom.css`,
                          ],
                          scanStyles: false,
                        })
                      })
                    })
                  })
                } else {
                  this.$message.warning('没有相关订单')
                  return false
                }
              }
            },
          },
        ],
        extraButtons: [
          {
            text: '发货',
            show: (row) => !row.partnerName && row.statusBackend === 'SEND',
            atClick: (row) => {
              this.$refs.crud.$refs.extraDialog[0].show(row)
              return false
            },
          },
          {
            text: '退款',
            show: (row) => row.paymentTime && row.statusBackend !== 'REFUND',
            atClick: async (row) => {
              try {
                await this.$elBusUtil.confirm('确定要退款吗？')
                const { code } = await this.$elBusHttp.request(
                  'flower/api/order/refund',
                  {
                    params: {
                      id: row.id,
                    },
                  }
                )
                if (code === 0) {
                  this.$message.success('退款成功')
                }
              } catch (e) {
                return false
              }
            },
          },
        ],
        extraDialogs: [
          {
            title: '发货',
            hiddenReverseItems: [],
            form: [
              {
                label: '快递号：',
                id: 'deliveryNo',
                type: 'input',
                rules: {
                  required: true,
                  message: '请输入快递号',
                  trigger: 'blur',
                },
              },
            ],
            atConfirm: async (val) => {
              const { code } = await this.$elBusHttp.request(
                'flower/api/order/list/send',
                { method: 'post', data: val }
              )
              if (code === 0) {
                this.$message.success('发货成功')
              }
            },
          },
        ],
      },
    }
  },
  head() {
    return {
      title: '订单列表',
    }
  },
  watch: {
    checkedLevelDown() {
      this.$refs.crud.search()
    },
  },
  methods: {
    getDistrict(row) {
      return `${row.customerProvince || ''}${row.customerCity || ''}${
        row.customerRegion || ''
      }`
    },
  },
}
</script>

<style lang="scss" scoped>
#print-container {
  width: 190mm;
  position: absolute;
  top: 9999px;
  left: 9999px;
}
</style>
